from logging import getLogger

from django.core.cache import cache
from rest_framework import response
from rest_framework.views import APIView

from ..services.user_sms_service import UserSmsService
from ..utils.custom_response import util_response
from ..utils.model_handle import parse_data

logger = getLogger('log')


class ShortMessageLogin(APIView):

    # 短信验证码校验
    def sms_login(self):
        # 1. 电话和手动输入的验证码
        params = parse_data(self)
        phone = params.get('phone')
        code = params.get('code')
        login_code = params.get('login_code', None)
        sso_serve_id = params.get('sso_serve_id', None)
        if code is None:
            return util_response(err=4002, msg="验证码不能为空")
        cache_code = cache.get(phone)
        if code == cache_code:
            app = UserSmsService()
            err, data = app.phone_login(phone=phone, login_code=login_code, sso_serve_id=sso_serve_id, detail_params=params)
            if err == 0:
                return util_response(data=data)
            else:
                logger.error('---短信登录错误：' + err + '---')
                return util_response(msg=err, err=6004)
        else:
            return util_response(err=4002, msg="验证码错误")

    # 短信验证码校验
    def post(self, request):
        # 1. 电话和手动输入的验证码
        phone = request.POST.get('phone')
        code = request.POST.get('code')
        login_code = request.data.get('login_code', None)
        sso_serve_id = request.data.get('sso_serve_id', None)

        # params = parse_data(self)
        print(self.request.POST.get('phone'))
        if code is None:
            res = {
                'err': 4002,
                'msg': '验证码不能为空',
            }
            return response.Response(data=res, status=None, template_name=None)
        cache_code = cache.get(phone)
        if code == cache_code:
            app = UserSmsService()
            err, data = app.phone_login(phone=phone, login_code=login_code, sso_serve_id=sso_serve_id)
            if err == 0:
                return util_response(data=data)
            else:
                return util_response(msg=err, err=6004)
        else:
            res = {
                'err': 4002,
                'msg': '验证码错误',
            }
            return response.Response(data=res, status=None, template_name=None)


class MyApiError(Exception):
    def __init__(self, message, err_code=4010):
        self.msg = message
        self.err = err_code

    def __str__(self):
        # repr()将对象转化为供解释器读取的形式。可省略
        return repr(self.msg)



# _*_coding:utf-8_*_

import os, logging, time, json, copy
from datetime import datetime, timedelta
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import response
from rest_framework import serializers
from rest_framework import generics
from rest_framework.permissions import IsAuthenticated, IsAdminUser, AllowAny
from django.db.models import Q
from django.db.models import F

from .models import *

logger = logging.getLogger(__name__)


def make_code():
    now = datetime.now()
    date_str = now.strftime('%Y%m%d')
    time_str: str = str(time.time())[5:10]
    ms_str = str(now)[-6:-3]
    code = '%s%s%s' % (date_str, time_str, ms_str)
    return code


# 获取用户信息
class UserLogin(APIView):  # 或继承APIView UpdateAPIView
    permission_classes = (AllowAny,)
    model = BaseInfo
    queryset = BaseInfo.objects.all()
    params = None

    def get(self, request, *args, **kwargs):
        self.params = request.query_params  # 返回QueryDict类型

        if 'account' not in self.params or self.params['account'].isspace():
            return Response({'err': 1001, 'msg': '缺少account', })
            # raise MyApiError("缺少account")
        if 'password' not in self.params or self.params['password'].isspace():
            return Response({'err': 1002, 'msg': '缺少password', })

        result = self.queryset.filter(Q(id=self.params['account']))
        response_data = result.values(
            'platform_uid',
            'platform',
            'user_name',
            'full_name',
            'phone',
            'email',
            'wechat',
            'user_info'
        )

        res = {
            'err': 0,
            'msg': 'OK',
            'data': response_data,
            'request': self.params,
            # 'serializer': serializer.data,
        }

        # try:

            # return super(UserLogin, self).patch(request, *args, **kwargs)
        #
        # except SyntaxError:
        #     print(">SyntaxError:")
        #     res = {
        #         'err': 4001,
        #         'msg': '语法错误'
        #     }
        # except LookupError:
        #     res = {
        #         'err': 4002,
        #         'msg': '无效数据查询'
        #     }
        # # 这里 result是一个类的对象，要用result.属性名来返回
        # except Exception as valueError:
        #     print('>Exception:', valueError)
        #     res = {
        #         'err': 4003,
        #         'msg': valueError.args
        #
        #     }
        # except:
        #     res = {
        #         'err': 4009,
        #         'msg': '未知错误'
        #     }

        return response.Response(res)


class UserInfoSerializer(serializers. ModelSerializer):
    class Meta:
        model = BaseInfo
        exclude = ['user_name', 'full_name',]


class UserInfo(generics.UpdateAPIView):  # 或继承(APIView)
    queryset = BaseInfo.objects.all()
    permission_classes = (AllowAny,)  # 允许所有用户 (IsAuthenticated,IsStaffOrBureau)
    serializer_class = UserInfoSerializer
    params = None

    def get(self, request, *args, **kwargs):
        self.params = request.query_params  # 返回QueryDict类型

        if 'uid' not in self.params or self.params['uid'].isspace():
            return Response({'err': 0, 'msg': '缺少uid', 'data': [], 'request': self.params, })

        base_info = BaseInfo.objects.all().filter(Q(id=self.params['uid']))
        data = base_info.values(
            'platform_uid',
            'platform',
            'user_name',
            'full_name',
            'phone',
            'email',
            'wechat',
            'user_info'
        )

        return Response({
            'err': 0,
            'msg': 'OK',
            'data': data,
            'request': self.params,
            # 'serializer': serializer.data,
        })
        # return super(UserLogin, self).patch(request, *args, **kwargs)


class MyApiError(Exception):
    def __init__(self, message):
        self.msg = message

    def __str__(self):
        # repr()将对象转化为供解释器读取的形式。可省略
        return repr(self.msg)


# encoding: utf-8
"""
@project: djangoModel->user_permission_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户权限服务
@created_time: 2022/8/23 9:33
"""
from django.db.models import F

from ..utils.j_dict import JDict
from ..models import BaseInfo, PermissionValue, Group


class PermissionService():
    __user_dict = None
    __is_err = False
    __err_message = None

    @staticmethod
    def get_group_user(user_id):
        inner_group = BaseInfo.objects.filter(id=user_id) \
            .annotate(parent_group_id=F("user_group__parent_group"))
        inner_group = inner_group.values('user_group_id', "parent_group_id").first() if inner_group else {'user_group_id': -1, "parent_group_id": -1}
        child_group_ids = [i['id'] for i in list(Group.objects.filter(parent_group=inner_group['user_group_id']).values("id"))]
        res_set = {
            "GROUP_INSIDE": [i['id'] for i in list(BaseInfo.objects.filter(user_group_id=inner_group['user_group_id']).values("id"))],
            "GROUP_PARENT": [i['id'] for i in list(BaseInfo.objects.filter(user_group_id=inner_group['parent_group_id']).values("id"))],
            "GROUP_CHILDREN": [i['id'] for i in list(BaseInfo.objects.filter(user_group_id__in=child_group_ids).values("id"))],
            "GROUP_OUTSIDE": []
        }
        return res_set, None

    @staticmethod
    def get_user_group_permission(user_id, module=None, feature="USER_GROUP", type=None):
        try:
            # 获取用户的权限
            params = {k: v for k, v in {"module": module, "feature": feature}.items() if v}
            permission_set = BaseInfo.objects.filter(id=user_id).annotate(user_permission_id=F("user_group__permission_id"))
            if not permission_set:  # 用户ID不存在
                return {}, None
            permission_dict = permission_set.values("user_permission_id").first()
            if not permission_dict['user_permission_id']:  # 没有用户组配置权限值
                return {}, None
            params.setdefault("permission_id", permission_dict['user_permission_id'])
            values = list(PermissionValue.objects.filter(**params).values(
                "module", "permission_value", "relate_value", "ban_view", "ban_edit", "ban_add", "ban_delete", "is_ban", "is_system", 'is_enable'
            ))
            if not values:  # 有权限ID但是没有 配置对应权限值
                return {}, None
            # 权限值
            res = JDict({})
            group_user, err = PermissionService.get_group_user(user_id)
            for item in values:
                item_copy = JDict(item)
                module_name = item_copy.pop('module')
                res.setdefault(module_name, {})

                current_module = getattr(res, module_name)
                current_module.setdefault('relate_value', item_copy.pop("relate_value"))

                permission_value = item_copy.pop('permission_value')
                item_copy["user_list"] = group_user[permission_value]
                current_module.setdefault(permission_value, item_copy)
            res = res[module] if module else res
            return res, None
        except Exception as e:
            print("msg:" + str(e) + "line:" + str(e.__traceback__.tb_lineno))
            return None, "msg:" + str(e) + "line:" + str(e.__traceback__.tb_lineno)



# encoding: utf-8
"""
@project: djangoModel->user_statistics
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户统计API
@created_time: 2022/11/3 10:48
"""
from rest_framework.views import APIView

from ..services.user_statistics_service import UserStatisticsService
from ..utils.custom_response import util_response


class UserStatisticsAPI(APIView):
    def get(self, request):
        data, err = UserStatisticsService.user_statistics()
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# _*_coding:utf-8_*_

import os, logging, time, json, copy
import re
from datetime import datetime, timedelta

from django.forms import model_to_dict
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import response
# from rest_framework import serializers
from rest_framework import generics
from rest_framework.permissions import IsAuthenticated, IsAdminUser, AllowAny
from django.utils.translation import gettext as tr

from xj_user.models import BaseInfo, DetailInfo
from xj_user.services.user_service import UserService, UserInfoSerializer

logger = logging.getLogger(__name__)


def make_code():
    now = datetime.now()
    date_str = now.strftime('%Y%m%d')
    time_str: str = str(time.time())[5:10]
    ms_str = str(now)[-6:-3]
    code = '%s%s%s' % (date_str, time_str, ms_str)
    return code


# 获取用户信息
class UserInfo(generics.UpdateAPIView):  # 或继承(APIView)
    permission_classes = (AllowAny,)  # 允许所有用户 (IsAuthenticated,IsStaffOrBureau)
    serializer_class = UserInfoSerializer  # 继承generics.UpdateAPIView需要设置此属性覆盖get_serializer_class()方法
    queryset = BaseInfo.objects.all()  # 继承generics.UpdateAPIView需要设置此属性覆盖get_queryset()方法
    # lookup_field = 'id'
    params = None

    def get(self, request, *args, **kwargs):
        self.params = request.query_params  # 返回QueryDict类型
        # print("> params:", self.params)
        # print("> language:", request.LANGUAGE_CODE)

        try:
            token = self.request.META.get('HTTP_AUTHORIZATION', None)
            print("> token:", token)

            token_serv, error_text = UserService.check_token(token)
            print("> token_serv:", token_serv)
            if error_text:
                raise MyApiError(error_text, 6010)
            account = token_serv['account']

            account_serv, error_text = UserService.check_account(account)
            if error_text:
                raise MyApiError(error_text, 6020)
            # print("> account_serv:", account_serv)
            user_info = account_serv
            user_info['platform_id'] = token_serv['platform_id']
            # avatar = None
            # detailinfo = DetailInfo.objects.filter(user_id=user_info['user_id']).first()
            # if detailinfo:
            #     detailinfo = model_to_dict(detailinfo)
            #     avatar = detailinfo['avatar']
            # user_info['avatar'] = avatar
            res = {
                'err': 0,
                'msg': 'OK',
                'data': user_info,
                # 'tr': tr('HELLO'),
                # 'request': self.params,
                # 'serializer': serializer.data,
            }

        except SyntaxError:
            # print("> SyntaxError:")
            res = {
                'err': 4001,
                'msg': '语法错误'
            }
        except LookupError:
            res = {
                'err': 4002,
                'msg': '无效数据查询'
            }
        # 这里 error是一个类的对象，要用error.属性名来返回
        except Exception as error:
            res = {
                'err': error.err if hasattr(error, 'err') else 4000,  # 发生系统异常时报4000
                'msg': error.msg if hasattr(error, 'msg') else error.args,  # 发生系统异常时捕获error.args
            }
            if not hasattr(error, 'err'):
                res['file'] = error.__traceback__.tb_frame.f_globals["__file__"],  # 发生异常所在的文件
                res['line'] = error.__traceback__.tb_lineno,  # 发生异常所在的行数
        except:
            res = {
                'err': 4999,
                'msg': '未知错误'
            }

        # return super(UserLogin, self).patch(request, *args, **kwargs)
        return response.Response(res)


class MyApiError(Exception):
    def __init__(self, message, err_code=4010):
        self.msg = message
        self.err = err_code

    def __str__(self):
        # repr()将对象转化为供解释器读取的形式。可省略
        return repr(self.msg)



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: CURD 工具
@created_time: 2022/6/15 14:14
"""

# ===========  工具方法 start =============
import json

from django.core import serializers
from django.core.paginator import Paginator

from .custom_response import *


# json 结果集返回
def parse_json(result):
    if not result is None:
        if type(result) is str:
            try:
                result = json.loads(result.replace("'", '"').replace('\\r', "").replace('\\n', "").replace('\\t', "").replace('\\t', ""))
            except Exception as e:
                return result
        if type(result) is list:
            for index, value in enumerate(result):
                result[index] = parse_json(value)
        if type(result) is dict:
            for k, v in result.items():
                result[k] = parse_json(v)
    return result


# 字段筛选并替换成别名
def format_params_handle(param_dict, filter_filed_list=None, remove_filed_list=None, alias_dict=None):
    # 转换的数据类型不符合，直接返回出去
    if not isinstance(param_dict, dict):
        raise Exception("param_dict 必须是字典格式")

    # 类型判断 过滤字段
    if filter_filed_list and isinstance(filter_filed_list, list):
        param_dict = {k: v for k, v in param_dict.copy().items() if k in filter_filed_list and v}

    # 类型判断， 剔除字段
    if remove_filed_list and isinstance(remove_filed_list, list):
        param_dict = {k: v for k, v in param_dict.copy().items() if not k in remove_filed_list and v}

    # 类型判断 字段转换
    if alias_dict and isinstance(alias_dict, dict):
        param_dict = {alias_dict.get(k, k): v for k, v in param_dict.copy().items()}

    return param_dict


# 请求参数解析
def parse_data(request, **kwargs):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
    method = request.method
    if method == "GET":
        data = request.GET
    elif method == "POST":
        if content_type == "application/json":
            data = json.loads(request.body)
        else:
            data = request.POST
    else:
        data = request.data
    return {k: v for k, v in data.items()}


# 模型解析
def parse_model(res_set, custom_serializers=None, is_simple=False, ):
    try:
        # 序列化转换执行
        if custom_serializers:
            custom_serializers_data = custom_serializers(res_set)
            return custom_serializers_data.data
        # 没有传序列化的时候默认进行，json序列化
        json_data = json.loads(serializers.serialize('json', res_set))
    except Exception as e:
        # 当使用了values()限制结果集会报错，使用list进行转换结果集
        return list(res_set)
    # 如果正常json过了，则进行格式化
    if not json_data:
        return None
    else:
        if is_simple:
            return json_data[0]['fields']
        else:
            res_set = []
            for i in json_data:
                fields = i['fields']
                fields['id'] = i['pk']
                res_set.append(fields)
            return res_set


# 序列换元组类型转换字典
def serializers_to_list(data):
    result = []
    for i in data:
        temp_dict = {}
        for k, v in i.items():
            if type(v) is tuple:
                serializers_to_list(v)
            temp_dict[k] = v
        result.append(temp_dict)
    return result


# =================== CURD ===============
def model_select(request, model, is_need_delete=False, json_parse_key=None):
    # 模型快速分页查询  分页+条件
    page = request.GET.get('page', 1)
    limit = request.GET.get('limit', 20)
    params = parse_data(request)
    if 'page' in params.keys():
        del params['page']
    if 'limit' in params.keys():
        del params['limit']

    if is_need_delete:
        params['is_delete'] = 0
    try:
        list_set = model.objects.filter(**params)
        count = model.objects.filter(**params).count()
    except Exception as e:
        return util_response("", 7557, e.__str__(), status.HTTP_400_BAD_REQUEST)
    # 分页数据
    limit_set = Paginator(list_set, limit)
    page_set = limit_set.get_page(page)
    # 数据序列化操作
    json_data = json.loads(serializers.serialize('json', page_set))
    final_res_dict = []
    for i in json_data:
        fields = i['fields']
        fields['id'] = i['pk']
        if not json_parse_key is None:
            fields[json_parse_key] = json.loads(fields[json_parse_key])
        final_res_dict.append(fields)
    # 数据拼装
    result = {'list': final_res_dict, 'limit': int(limit), 'page': int(page), 'count': count}
    return util_response(result, 0)


def model_create(request, model, validate):
    try:
        requestData = parse_data(request)
        if not validate is None:
            validator = validate(requestData)
            is_pass, error = validator.validate()
            if not is_pass:
                return util_response("", 7557, error)

        model.objects.create(**requestData)
    except Exception as e:
        return util_response("", 7557, e.__str__(), status.HTTP_400_BAD_REQUEST)
    return util_response()


def model_update(request, model, is_need_delete=False):
    # 模型修改
    from_data = parse_data(request)
    if not 'id' in from_data.keys():
        return util_response('', 7557, "ID不能为空")
    id = from_data['id']
    del from_data['id']
    if from_data == {}:
        return util_response('', 7557, "至少改点什么吧！！")
    # 是否存软删除条件
    if is_need_delete:
        res = model.objects.filter(id=id, is_delete=0)
    else:
        res = model.objects.filter(id=id)
    # 修改并返回
    if not res:
        return util_response('', 7557, "数据已不存在")
    try:
        res.update(**from_data)
        return util_response('', 0, "ok")
    except Exception as e:
        return util_response("", 7557, e.__str__(), status.HTTP_400_BAD_REQUEST)


def model_delete(request, model, is_real_delete=True):
    from_data = parse_data(request)
    if not 'id' in from_data.keys():
        return util_response('', 7557, "ID不能为空")
    if is_real_delete:
        res = model.objects.filter(id=from_data['id'])
        # res = model.objects.filter(**from_data)
        if not res:
            return util_response('', 7557, "数据已不存在")

        res.delete()
    else:
        from_data['is_delete'] = 0
        res = model.objects.filter(**from_data)
        if not res:
            return util_response('', 7557, "数据已不存在")
        res.update(is_delete=1)

    return util_response()



# _*_coding:utf-8_*_

# import os, logging, time, json, copy
import re
import jwt
from django.contrib.auth.hashers import make_password
from django.db.models import Q
from pathlib import Path
from main.settings import BASE_DIR
from rest_framework import response
from rest_framework.permissions import AllowAny
from rest_framework.views import APIView

from xj_role.services.user_group_service import UserGroupService
from xj_role.services.role_service import RoleService
from ..models import *
from ..services.user_service import UserService
from ..utils.model_handle import parse_data
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict


module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

jwt_secret_key = main_config_dict.jwt_secret_key or module_config_dict.jwt_secret_key or ""

# 管理员添加用户
class UserAdd(APIView):
    permission_classes = (AllowAny,)
    model = BaseInfo
    params = None

    def post(self, request, *args, **kwargs):
        self.params = parse_data(request)
        # 权限验证
        token = request.META.get('HTTP_AUTHORIZATION', '')
        token_serv, error_text = UserService.check_token(token)
        if error_text:
            raise MyApiError(error_text, 6010)
        # 添加逻辑
        try:
            account = str(self.params.get('account', ''))
            password = str(self.params.get('password', ''))
            phone = str(self.params.get('phone', ''))
            full_name = str(self.params.get('full_name', ''))
            nickname = str(self.params.get('nickname', ''))

            # 用户角色部门绑定
            user_role_list = self.params.get('user_role_list', None)
            user_group_list = self.params.get('user_group_list', None)

            # 边界检查
            if not account:
                raise MyApiError("account必填", 2001)

            if not password:
                raise MyApiError("password必填", 2003)

            if not full_name:
                raise MyApiError("full_name必填", 2008)

            # 账号类型判断
            if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
                account_type = 'phone'
            elif re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', account):
                account_type = 'email'
            elif re.match(r'^[A-z\u4E00-\u9FA5]+\w*$', account):
                account_type = 'username'
            else:
                raise MyApiError("账号必须是用户名、手机或者邮箱，用户名不能是数字开头", 2009)

            # 检查账号是否存在
            user_list = None
            if account_type == 'phone':
                user_list = BaseInfo.objects.filter(Q(phone=account))
            elif account_type == 'email':
                user_list = BaseInfo.objects.filter(Q(email=account))
            elif account_type == 'username':
                user_list = BaseInfo.objects.filter(Q(user_name=account))

            if user_list.count() and account_type == 'phone':
                raise MyApiError("手机已被注册: " + account)
            elif user_list.count() and account_type == 'email':
                raise MyApiError("邮箱已被注册: " + account)
            elif user_list.count() and account_type == 'username':
                raise MyApiError("用户名已被注册: " + account)

            # # 检查平台是否存在
            # platform_id = None
            # platform_set = Platform.objects.filter(platform_name__iexact=platform)
            # if not platform_set.count() == 0:
            #     platform_id = platform_set.first().platform_id

            base_info = {
                'user_name': account if account_type == 'username' else '',
                'full_name': full_name,
                "nickname": nickname,
                'phone': account if account_type == 'phone' else phone,
                'email': account if account_type == 'email' else '',
            }
            current_user = BaseInfo.objects.create(**base_info)
            token = jwt.encode({'account': account}, jwt_secret_key)
            auth = {
                'user_id': current_user.id,
                'password': make_password(password, None, 'pbkdf2_sha1'),
                'plaintext': password,
                'token': token,
            }
            Auth.objects.create(**auth)
            # 用户绑定权限和部门
            if user_group_list:
                UserGroupService.bind_user_group(current_user.id, user_group_list)

            if user_role_list:
                RoleService.bind_user_role(current_user.id, user_role_list)

            res = {
                'err': 0,
                'msg': '添加成功',
                'data': {"user_id": current_user.id},
            }

        except SyntaxError:
            res = {
                'err': 4001,
                'msg': '语法错误'
            }
        except LookupError:
            res = {
                'err': 4002,
                'msg': '无效数据查询'
            }
        except Exception as valueError:
            res = {
                'err': valueError.err if hasattr(valueError, 'err') else 4000,
                'msg': valueError.msg if hasattr(valueError, 'msg') else valueError.args,
            }
        except:
            res = {
                'err': 4999,
                'msg': '未知错误'
            }

        return response.Response(data=res, status=None, template_name=None, headers={"Authorization": token},
                                 content_type=None)


class MyApiError(Exception):
    def __init__(self, message, err_code=4010):
        self.msg = message
        self.err = err_code

    def __str__(self):
        # repr()将对象转化为供解释器读取的形式。可省略
        return repr(self.msg)



# _*_coding:utf-8_*_

from pathlib import Path
# import os, logging, time, json, copy
import re

# from django.db.models import Q
# from django.db.models import F
from django.contrib.auth.hashers import make_password
from django.db.models import Q
import jwt
# from rest_framework.response import Response
from rest_framework import response
# from rest_framework import serializers
# from rest_framework import generics
# from rest_framework.permissions import IsAuthenticated
# from rest_framework.permissions import IsAdminUser
from rest_framework.permissions import AllowAny
from rest_framework.views import APIView

from main.settings import BASE_DIR
from utils.custom_tool import parse_data
from xj_role.services.role_service import RoleService
from xj_role.services.user_group_service import UserGroupService
from xj_user.models import *
from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.utils.nickname_generate import gen_one_word_digit
# import random
from ..utils.custom_response import util_response
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict

module_dir = Path(__file__).parent.parent
root_config = JDict(JConfig.get_section(str(BASE_DIR) + '/config.ini', 'xj_user', encode='utf-8-sig'))
module_config = JDict(JConfig.get_section(str(module_dir) + '/config.ini', 'xj_user', encode='utf-8-sig'))


# from datetime import datetime, timedelta
# from django.contrib.auth.hashers import check_password
# from django.conf import settings

# logger = logging.getLogger(__name__)


# 用户注册
class UserRegister(APIView):
    permission_classes = (AllowAny,)
    model = BaseInfo
    params = None

    def post(self, request, *args, **kwargs):
        self.params = request.data  # 返回QueryDict类型
        token = None

        try:
            account = str(self.params.get('account', ''))
            password = str(self.params.get('password', ''))
            platform = str(self.params.get('platform', ''))
            full_name = str(self.params.get('full_name', ''))

            # 边界检查
            if not account:
                raise MyApiError("account必填", 2001)

            if not password:
                raise MyApiError("password必填", 2003)

            # if not platform:
            #     raise MyApiError("platform必填", 2007)

            if not full_name:
                raise MyApiError("full_name必填", 2008)

            # 检查平台是否存在
            # platform_set = Platform.objects.filter(platform_name__iexact=platform)
            # if platform_set.count() == 0:
            #     raise MyApiError("platform不存在平台名称："+platform, 2009)
            # platform_id = platform_set.first().platform_id

            # 账号类型判断
            if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
                account_type = 'phone'
            elif re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', account):
                account_type = 'email'
            elif re.match(r'^[A-z\u4E00-\u9FA5]+\w*$', account):
                account_type = 'username'
            else:
                raise MyApiError("账号必须是用户名、手机或者邮箱，用户名不能是数字开头", 2009)
            # print('> account_type:', account_type)

            # 检查账号是否存在
            user_list = None
            if account_type == 'phone':
                user_list = BaseInfo.objects.filter(Q(phone=account))
            elif account_type == 'email':
                user_list = BaseInfo.objects.filter(Q(email=account))
            elif account_type == 'username':
                user_list = BaseInfo.objects.filter(Q(user_name=account))
            # print("> user_list:", user_list)

            if user_list.count() and account_type == 'phone':
                raise MyApiError("手机已被注册: " + account)
            elif user_list.count() and account_type == 'email':
                raise MyApiError("邮箱已被注册: " + account)
            elif user_list.count() and account_type == 'username':
                raise MyApiError("用户名已被注册: " + account)

            base_info = {
                # 'platform_uid': round(random.random()*1000000000),
                # 'platform_id': platform_id,
                'user_name': account if account_type == 'username' else '',
                'phone': account if account_type == 'phone' else '',
                'email': account if account_type == 'email' else '',
                'full_name': full_name,
            }
            # print("> base_info:", base_info)
            current_user = BaseInfo.objects.create(**base_info)
            # print("> current_user:", current_user.id)

            # SECURITY WARNING: keep the secret key used in production secret!
            SECRET_KEY = 'django-insecure-l1$-m!u=!f9&o2$f(cm#dasus&a=5i1#+kh)090_=p%+==%9o1'
            JWT_SECRET_KEY = '@xzm2021!'
            c1 = root_config
            c2 = module_config
            jwt_secret_key = c1.jwt_secret_key or c2.jwt_secret_key or "@zxmxy2021!"
            token = jwt.encode({'account': account}, jwt_secret_key)
            auth = {
                'user_id': current_user.id,
                'password': make_password(password, None, 'pbkdf2_sha1'),
                'plaintext': password,
                # 'token': jwt.encode(current_user.id, settings.JWT_SECRET_KEY),
                'token': token,
            }
            # print("> auth:", auth)
            # print("> check_password:", check_password(password, 'pbkdf2_sha1$260000$jNjzstMtWofWN0tmUzfQwh$02D9zbyiPyYHZwb2OdrpmTwN8zI='))

            current_auth = Auth.objects.create(**auth)

            res = {
                'err': 0,
                'msg': '注册成功',
                'data': {
                    "user_id": current_user.id,
                    # "token": token,
                },
            }

        except SyntaxError:
            # print("> SyntaxError:")
            res = {
                'err': 4001,
                'msg': '语法错误'
            }
        except LookupError:
            res = {
                'err': 4002,
                'msg': '无效数据查询'
            }
        # 这里 result是一个类的对象，要用result.属性名来返回
        except Exception as valueError:
            # print('> Exception:', valueError)
            res = {
                'err': valueError.err if hasattr(valueError, 'err') else 4000,
                'msg': valueError.msg if hasattr(valueError, 'msg') else valueError.args,
                # 'request': self.params,
            }
        except:
            res = {
                'err': 4999,
                'msg': '未知错误'
            }

        # return super(UserLogin, self).patch(request, *args, **kwargs)

        # return super(UserLogin, self).patch(request, *args, **kwargs)
        headers = {
            "Authorization": token,
        }
        return response.Response(data=res, status=None, template_name=None, headers=headers, content_type=None)

    def put(self, request, *args, **kwargs):
        self.params = parse_data(request)
        detaili_list = DetailInfoService.transform_result(self.params)
        detaili = detaili_list[0]
        # 添加逻辑
        try:
            user_name = str(self.params.get('user_name', ''))
            full_name = str(self.params.get('full_name', ''))
            phone = str(self.params.get('phone', ''))
            nickname = str(self.params.get('nickname', ''))
            password = str(self.params.get('password', ''))
            # 用户角色部门绑定
            user_role_list = self.params.get('user_role_list', None)
            user_group_list = self.params.get('user_group_list', None)

            base_info = {}
            if user_name:
                base_info['user_name'] = user_name
                is_exists = BaseInfo.objects.filter(user_name=user_name).exists()
                if is_exists:
                    return util_response('', 7557, "用户名已存在")

            if full_name:
                base_info['full_name'] = full_name
            if nickname:
                base_info['nickname'] = nickname
            else:
                base_info['nickname'] = gen_one_word_digit()
            if phone:
                base_info['phone'] = phone

            current_user = BaseInfo.objects.create(**base_info)

            c1 = root_config
            c2 = module_config
            jwt_secret_key = c1.jwt_secret_key or c2.jwt_secret_key or "@zxmxy2021!"
            token = jwt.encode({'account': user_name}, jwt_secret_key)

            if password:
                auth = {
                    'user_id': current_user.id,
                    'password': make_password(password, None, 'pbkdf2_sha1'),
                    'plaintext': password,
                    'token': token,
                }
                Auth.objects.create(**auth)

            if detaili.get("user_name"):
                del detaili['user_name']
            if detaili.get("full_name"):
                detaili['real_name'] = detaili['full_name']
                del detaili['full_name']
            if detaili.get("nickname"):
                del detaili['nickname']
            if detaili.get("phone"):
                del detaili['phone']
            if detaili.get("email"):
                del detaili['email']
            if detaili.get("user_role_list"):
                del detaili['user_role_list']
            if detaili.get("user_group_list"):
                del detaili['user_group_list']
            detaili = DetailInfoService.transform_params(detaili)
            detailinfo = DetailInfo.objects.filter(user_id=current_user.id).first()
            if detailinfo:
                DetailInfo.objects.filter(user_id=current_user.id).update(**detaili)
            else:
                detaili['user_id'] = current_user.id
                DetailInfo.objects.create(**detaili)

            # 用户绑定权限和部门
            if user_group_list:
                UserGroupService.bind_user_group(current_user.id, user_group_list)

            if user_role_list:
                RoleService.bind_user_role(current_user.id, user_role_list)

            res = {
                'err': 0,
                'msg': '创建成功',
                'data': {"user_id": current_user.id},
            }

        except SyntaxError:
            res = {
                'err': 4001,
                'msg': '语法错误'
            }
        except LookupError:
            res = {
                'err': 4002,
                'msg': '无效数据查询'
            }
        except Exception as valueError:
            res = {
                'err': valueError.err if hasattr(valueError, 'err') else 4000,
                'msg': valueError.msg if hasattr(valueError, 'msg') else valueError.args,
            }
        except:
            res = {
                'err': 4999,
                'msg': '未知错误'
            }

        return response.Response(data=res, status=None, template_name=None, headers={"Authorization": token},
                                 content_type=None)


class MyApiError(Exception):
    def __init__(self, message, err_code=4010):
        self.msg = message
        self.err = err_code

    def __str__(self):
        # repr()将对象转化为供解释器读取的形式。可省略
        return repr(self.msg)
